#include <sys/types.h>

#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "iconv.h"

extern char *__progname;

static __dead void usage(void);

static __dead void
usage(void)
{
	fprintf(stderr, "usage: %s -f fromcode -t tocode\n", __progname);
	exit(1);
}

int
main(int argc, const char **argv)
{
	iconv_t ic;
	char buf[1024], obuf[2048];
	char *in, *out;
	size_t inlen, outlen, nrd, wrsz, nwr, total;
	char *from, *to;
	int ch, toquit;

	from = to = NULL;

	while ((ch = getopt(argc, argv, "f:t:")) != -1) {
		switch (ch) {
		case 'f':
			from = optarg;
			break;

		case 't':
			to = optarg;
			break;

		default:
			usage();
			/* NOTREACHED */
		}
	}

	ic = iconv_open(to, from);
	if (ic == (iconv_t)-1)
		errx(1, "iconv_open");

	inlen = 0;
	for (;;) {
		if (inlen > 0) {
			in = buf;
			out = obuf;
			outlen = sizeof(obuf);
			iconv(ic, &in, &inlen, &out, &outlen);
			toquit = (errno == EILSEQ);
			if (in != buf)
				memmove(buf, in, inlen);
			if (out != obuf) {
				total = sizeof(obuf) - outlen;
				wrsz = 0;
				while (wrsz < total) {
					nwr = fwrite(obuf + wrsz, 1,
					    total - wrsz, stdout);
					if (ferror(stdout) != 0)
						errx(1, "fwrite");
					wrsz += nwr;
				}
			}
			if (toquit)
				errx(1, "iconv");
		}

		if (inlen == sizeof(buf))
			errx(1, "not enoght space for input buffer");

		nrd = fread(buf + inlen, 1, sizeof(buf) - inlen, stdin);
		if (nrd == 0) {
			if (ferror(stdin) != 0)
				errx(1, "fread");
			break;	/* EOF */
		}

		inlen += nrd;
	}

	iconv_close(ic);

	return (0);
}
